##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
# 
##############################################################################
__doc__ = """ Default Implementation for Managed Threads

$Id: ManagedThread.py,v 1.1.2.1 2002/12/11 10:19:51 jack-e Exp $"""

import sys
from threading import Thread, Event, Lock

import zLOG

from Transaction import get_transaction
from Zope.App.Tasks.IManagedThread import IManagedThread
from Zope.Exceptions.ExceptionFormatter import format_exception

THREAD_WORKING   = 0
THREAD_FINISHED  = 1
THREAD_SLEEPING  = 2
THREAD_EXCEPTION = 3


class ManagedThread(object):

    __implements__ = IManagedThread

    def __init__(self):
        self._zodb = None
        self.args = ()
        self.kw = {}
        self.running = 0
        self.shutdown_asap = 0
        self.thread_wakeup = Event()
        self.write_lock = Lock()
        
        self._thread = None

    # security -> principal here 
    def start(self, _zodb, *args, **kw):
        self._zodb = _zodb
        self.args = args
        self.kw = kw
        self._thread = t = Thread(target=self.run)
        t.setDaemon(1)
        t.start()
        
    def stop(self, timeout=0):
        if not self.isRunning():
            return
        self.shudown_asap = 1
        self.thread_wakeup.set()
        self._thread.join(timeout)
        return self.isRunning()

    def isRunning(self):
        if self._thread is None:
            return 
        return self.running and self._thread.isAlive()


    def run(self):
        # switch to new securitymanager here
        self.running = 1
        work = apply(self.doWork, self.args, self.kw)
        status = THREAD_WORKING
        result = None
        while not self.shutdown_asap and not status == THREAD_FINISHED:

            try:
                status, result = work.next()
            except StopIteration:
                break
            except:
                status, result = THREAD_EXCEPTION, sys.exc_info()
            
            if status == THREAD_EXCEPTION:
                get_transaction().abort()
                tb_text = ''.join(
                        format_exception(*result, **{'as_html': 0}))
                zLOG.LOG("ThreadManager",zLOG.ERROR,"Exception in Thread: %s" % tb_text)
                break
            elif status == THREAD_SLEEPING:
                self.thread_wakeup.wait(float(result))

            if self.thread_wakeup.isSet():
                self.thread_wakeup.clear()

        # if thread was working, cleanup
        if status == THREAD_WORKING and callable(result):
            result()
                
        self.running = 0

        
    def doWork(self, *args, **kw):
        ## this is implemented a generator function
        ## that yields a Tuple (result, value) for ThreadManagement
        ## yield THREAD_WORKING, cleanup -> Do more work, if interrupted use cleanup
        ## yield THREAD_FINISHED, None -> No more work
        ## yield THREAD_EXCEPTION, exc_value -> There was an Exception
        ## yield THREAD_SLEEPING, sleeptime -> Thread wants to sleep
        raise NotImplementedError, "Overwrite this Method to do some work."
