##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
# 
##############################################################################
__doc__ = """ Implementation for ThreadManager Utility

$Id: ThreadManager.py,v 1.1.2.1 2002/12/11 10:19:51 jack-e Exp $"""

from thread import start_new_thread, allocate_lock

from Zope.App.Tasks.IThreadManager import IThreadManager
from Zope.App.Tasks.IManagedThread import IManagedThread

class ThreadManager:

    __implements__ = IThreadManager

    def __init__(self, _zodb):
        self._zodb = _zodb
        self.threads = {}
        self.mgmt_lock = allocate_lock()

    # security-context !!!
    def add(self, name, thread_object, *args, **kw):
        if self.threads.has_key(name):
            raise ValueError, "Thread with name %s already exists." % name
        if not IManagedThread.isImplementedBy(thread_object):
            raise TypeError, "Only ManagedThreads can be added."
        
        self.mgmt_lock.acquire()
        self.threads[name] = thread_object
        self.mgmt_lock.release()

        thread_object.start(self._zodb, *args, **kw)

    def remove(self, name, timeout=0):
        if not self.threads.has_key(name):
            raise ValueError, "Thread with name %s unknown." % name

        self.mgmt_lock.acquire()
        thread_object = self.threads[name]
        del self.threads[name]
        self.mgmt_lock.release()

        thread_object.stop(timeout)
        return not thread_object.isRunning()

    def get(self, name):
        return self.threads.get(name,None)

    def names(self):
        return self.threads.keys()
