##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
# 
##############################################################################
"""
This module starts a new Thread for producing TimeEvents. 

$Id: TimeEventProducer.py,v 1.1.2.1 2002/12/09 16:17:59 jack-e Exp $
"""
from Transaction import get_transaction
from Zope.App.ZopePublication.ZopePublication import ZopePublication
from Zope.App.Traversing import traverse
from Zope.ComponentArchitecture import getService

from Zope.App.Tasks.TimeEvent import TimeEvent

from datetime import datetime, timedelta
import time


time_units = {'sec':1,
              'min':60,
              'hour':3600,
              'day':86400}


class TimeEventPublication:

    root_name = ZopePublication.root_name

    def __init__(self, db):
        self.db = db
        self._conn = None


    def beforeCall(self):
        get_transaction().begin()


    def getApplication(self):
        self._conn = conn = self.db.open()
        root = conn.root()
        app = root.get(self.root_name, None)
        
        if app is None:
            raise SystemError, "Zope Application Not Found"        

        return app


    def getEventService(self, app, path):
        place = traverse(app, path)
        event_service = getService(place,"Events")
        return event_service


    def publishTimeEvent(self, event_service, event_time=datetime.now()):
        event_service.publish(TimeEvent(event_time))
        # print "published TimeEvent %s" % event_time


    def afterCall(self):
        get_transaction().commit()
        self._conn.close()



def startTimeEventProducer(_zodb, *args):
    """TimeEventProducer producing regular TimeEvents

    A TimeEventProducer takes a resolution and unit and produces
    TimeEvents according to the specified resolution.
    """
    config = {}
    for arg in args:
        key,val = arg.split('=')
        config[key] = val

    resolution = int(config.get('resolution', 60))
    if resolution < 1:
        resolution = 1

    unit = config.get('unit', 'sec')
    if not unit in time_units.keys():
        # unkown unit
        return
    unit_val = time_units[unit]

    path = config.get('path', '/')

    
    # try to fire at first time feasible for unit
    now = datetime.now()
    if unit == 'sec':
        delta = datetime(now.year, now.month, now.day, now.hour, now.minute, now.second+1) - now
    elif unit == 'min':
        delta = datetime(now.year, now.month, now.day, now.hour, now.minute+1) - now
    elif unit == 'hour':
        delta = datetime(now.year, now.month, now.day, now.hour+1) - now
    elif unit == 'day':
        delta = datetime(now.year, now.month, now.day+1) - now
    else:
        delta = timedelta(0, 0, 0)

    time.sleep(delta.seconds + delta.microseconds/1000000)


    while 1:
        try:
            pub = TimeEventPublication(_zodb)
            app = pub.getApplication()
            pub.beforeCall()
            evt_svc = pub.getEventService(app, path)

            now = datetime.now()
            pub.publishTimeEvent(evt_svc, now)
            pub.afterCall()
        except:
            try:
                pub.afterCall()
            except:
                pass


        time.sleep(resolution*unit_val)

    # cleanup ??
